################################################################################
############## Scorecards function to load saved data files ####################
################################################################################


SC_load_metrics <- function(recipe, m = sprintf("%02d", 1:12)) {
  
  ## recipe: read one recipe to know your model and info
  ## m: start_months to load from metrics outputs files
  
  start.year <- recipe$Analysis$Time$hcst_start ## 1993
  end.year <- recipe$Analysis$Time$hcst_end ## 2016
  var.name <- recipe$Analysis$Variables$name
  system.filename <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5'
  reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
  var.filename <- recipe$Analysis$Variables$name ## 'tas'
  period <- paste0(start.year, "-", end.year)
  
  metrics <- initialfunction_read_data(m, 
                                       input.path = recipe$Run$output, 
                                       system.filename = system.filename,
                                       reference.filename = reference.filename,
                                       var.filename = var.filename,
                                       hindcasts.filename = period, 
                                    #   m = m, 
                                       mod.pressure = NULL, 
                                       unitconversionfactor = initialfunction_conversionfactor(var.name, system.filename), 
                                       metric = NULL)
    
  lon <- initialfunction_latlon_values(input.path = recipe$Run$output, 
                                       system.filename = system.filename,
                                       reference.filename = reference.filename,
                                       var.filename = var.filename,
                                       hindcasts.filename = period, 
                                       mod.pressure = NULL, 
                                       'longitude')

   lat <- initialfunction_latlon_values(input.path = recipe$Run$output, 
                                        system.filename = system.filename,
                                        reference.filename = reference.filename,
                                        var.filename = var.filename,
                                        hindcasts.filename = period,
                                        mod.pressure = NULL,
                                        'latitude')

  data <- append(metrics, list(lon = lon, lat = lat))
  return(data)
}



##### Load data metrics files #####

initialfunction_read_data <- function(start.months, input.path, system.filename, reference.filename, var.filename, hindcasts.filename, m, mod.pressure, unitconversionfactor, metric){
  
  bias <- corr <- rpss <- crpss <- list()
  for(m in start.months){
    ## open netCDF file:
    if(m=="01" & system.filename == "ukmos14"){hindcasts.filename<-"1994-2016"}else{hindcasts.filename<-"1993-2016"} ## considering missing startdate
    
    print(paste0("Loading netCDF file: ", paste0(input.path, "/", system.filename, "/", var.filename, 
                                                 "/scorecards_", system.filename, "_", reference.filename, "_",
                                                 var.filename, "_", hindcasts.filename, "_s", m, mod.pressure, ".nc")))
    
    nc <- nc_open(paste0(input.path, "/", system.filename, "/", var.filename, 
                         "/scorecards_", system.filename, "_", reference.filename, "_", 
                         var.filename, "_", hindcasts.filename, "_s", m, mod.pressure,".nc"))
    
    ## load data from variables:
    mm <- as.numeric(m)
    ## Temporarily setting bias to enscorr since not currently available
    bias[[mm]] <- ncvar_get(nc, "enscorr")
    corr[[mm]] <- ncvar_get(nc, "enscorr")
    rpss[[mm]] <- ncvar_get(nc, "rpss")
    crpss[[mm]] <- ncvar_get(nc, "crpss")
    
    
    ## set to NA values of bias that are exactly 0 because they correspond to mask values 
    # (as a difference between tho numbers with many decimales can be zero only very rarely). 
    # Set to NA correlation values or correlation that are infinite. 
    # Set to NA values of rps.clim and crps.clim that are exactly 0 because they correspond to mask values. 
    # In this case values are not a difference as for bias, so 0 values may be right values of rps.clim instead of mask values. 
    # However, an RPS.clim of exactly 0 means perfect climatological forecasts, and this we know it is not possible, 
    # only explanation is that input data has some issues, for example it is constant data, as in case of Greenland and Antarctica. 
    # Thus, these grid points can be added to points of the mask.
    
    bias[[mm]][which(bias[[mm]] == 0)] <- NA 
    corr[[mm]][which(is.infinite(corr[[mm]]))] <- NA 
    
    # Data transformation for visualization (special cases):
    if(unitconversionfactor != 1) bias[[mm]] <- bias[[mm]] * unitconversionfactor
    if(var.filename == "snow_depth"){
      bias[[mm]][which(bias[[mm]] < -100)] <- NA ## remove points close to Greenland and Antarctica where sometimes the values of the bias are close to -10 m
      bias[[mm]][which(bias[[mm]] > 100)] <- NA ## remove points close to Greenland and Antarctica where sometimes the values of the bias are close too high
    }
    ## close netCDF:
    nc_close(nc)
  }              
  
  if(is.null(metric)) {
    data <- list(bias = bias, corr = corr, rpss = rpss, crpss = crpss)
  } else if (metric == "bias") {
    data <- bias
  } else if(metric == "corr") {
    data <- corr
  } else if(metric == "rpss") {
    data <- rpss
  } else if(metric == "crpss") {
    data <- crpss
  } else {
    error("Request your metric")
  } 
  return(data)
}

##### Units conversion function #####

initialfunction_conversionfactor <- function(var.name, system.filename) {
  # transformation of units for data that need transformation for correct visualization
  unitconversionfactor = 1
  if(var.name == "Total precipitation" || var.name == "Snowfall" || var.name == "Runoff" || var.name == "Evaporation"){
    unitconversionfactor = 10^8
  }
  if(var.name == "Snow depth" || var.name == "East-west surface stress rate of accumulation" || var.name == "North-south surface stress rate of accumulation"){
    unitconversionfactor = 10^2
  }
  if(var.name == "Specific humidity"){
    unitconversionfactor = 10^6
  }
  if(var.name == "Snow depth" && system.filename == "meteo_frances7"){
    unitconversionfactor = 10^-2
  }
  
  return (unitconversionfactor)
}



##### Get lat and lon values #####

initialfunction_latlon_values <- function(input.path, system.filename, reference.filename, var.filename, hindcasts.filename, mod.pressure, latlon){
  ## open netCDF file:
  m <- "02"
  nc <- nc_open(paste0(input.path, "/", system.filename, "/", var.filename, 
                       "/scorecards_", system.filename, "_", reference.filename, "_", 
                       var.filename, "_", hindcasts.filename, "_s", m, mod.pressure,".nc"))
  
  coords <- ncvar_get(nc, latlon)
  
  return(coords)
  
} 
