################################################################################
######### Script to load scorecard metrics data for visualization ##############
################################################################################

transform_loaded_metrics <- function(recipe, loaded_metrics){
    plevel <- "" # to do: read from recipe
    start.year <- recipe$Analysis$Time$hcst_start ## 1993
    end.year <- recipe$Analysis$Time$hcst_end ## 2016
    var.name <- recipe$Analysis$Variables$name
    system.filename <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5'
    reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
    var.filename <- recipe$Analysis$Variables$name ## 'tas'
    period <- paste0(start.year, "-", end.year)
    reference.name <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_name ## 'ERA5'
    
    ## Define pressure levels if necessary
    if(plevel==""){
      mod.pressure <- ""
      mod.pressure.title <- ""
    }else{
      plevel <- as.integer(plevel)
      mod.pressure <- paste0("-p", plevel)
      mod.pressure.title <- paste0(" at ", plevel, " hPa")
    }
    
    ## Convert units
    unitconversionfactor <- initialfunction_conversionfactor(var.name, system.filename)
    if (unitconversionfactor != 1){var.unit <- paste0(gsub("e-0", "e-", gsub("+0", "", toString(formatC(1/unitconversionfactor, format="e", digits=0)))), " ", var.unit)}
    n.dec <- initialfunction_roundbias(var.name, plevel)
    
    ## Define regions
    region.names <- initialfunction_regionnames_and_borders()[,1]
    region.names.figure <- initialfunction_regionnames_and_borders()[,2]
    lat.min <- initialfunction_regionnames_and_borders()[,3]
    lat.max <- initialfunction_regionnames_and_borders()[,4]
    lon.min <- c() #initialfunction_regionnames_and_borders()[,5]
    lon.max <- c() #initialfunction_regionnames_and_borders()[,6]
    n.regions <- length(region.names)
    
    ## Suffix with hindcast period written in the filename:
    hindcasts.filename <- initialfunction_hindcastsperiod(reference.name, start.year, end.year, "hind")
    ref.caption <- initialfunction_hindcastsperiod(reference.name, start.year, end.year, "ref")
    
    ## Load metrics from the 12 netCDF files of the chosen forecast system, one for each start month:
    start.months <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
    n.start.months <- length(start.months)
    
    ## Define metrics
    bias <- loaded_metrics$bias
    corr <- loaded_metrics$corr
    rpss <- loaded_metrics$rpss
    crpss <- loaded_metrics$crpss
    
    ## Load lat and lon values
    lat.netCDF <- loaded_metrics$lat
    lon.netCDF <- loaded_metrics$lon
    
    ## position of lat and lon min and max values inside the netCDF
    pos.lat.min <- pos.lat.max <- pos.lon.min <- pos.lon.max <- c()
    for(i in 1:n.regions){
      if(is.null(lat.min)) pos.lat.min[i] <- which(lat.netCDF == min(lat.netCDF)) else pos.lat.min[i] <- initialfunction_position_latlon(as.integer(lat.min[i]), lat.netCDF)
      if(is.null(lat.max)) pos.lat.max[i] <- which(lat.netCDF == max(lat.netCDF))  else pos.lat.max[i] <- initialfunction_position_latlon(as.integer(lat.max[i]), lat.netCDF)
      if(is.null(lon.min)) pos.lon.min[i] <- which(lon.netCDF == min(lon.netCDF)) else pos.lon.min[i] <- initialfunction_position_latlon(as.integer(lon.min[i]), lon.netCDF)
      if(is.null(lon.max)) pos.lon.max[i] <- which(lon.netCDF == max(lon.netCDF))  else pos.lon.max[i] <- initialfunction_position_latlon(as.integer(lon.max[i]), lat.netCDF)
    }
    
    ## Forecast months to extract
    my.for.months <- 1:6 ## select the first 6 forecast months 
    n.for.months <- length(my.for.months)
    
    ## Empty arrays with the score values to plot:
    mean.bias <- mean.corr <- mean.crpss <- mean.rpss <- array(NA, c(n.start.months, n.regions, n.for.months))
    
    for(i in 1:n.regions){
      for(m in start.months){
        mm <- as.numeric(m)
        
        ## Select only scores inside the target region: [lon, lat, n.for.months]
        bias.region <- bias[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.for.months]
        corr.region <- corr[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.for.months]
        crpss.region <- crpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.for.months]
        rpss.region <- rpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.for.months]
        
        ## Replace values with (-)Inf with NA or the average will be (-)Inf too: (but infinite values should be absent in theory)
        rpss.region[which(is.infinite(rpss.region))] <- NA 
        crpss.region[which(is.infinite(crpss.region))] <- NA 
        
        ## Average score values inside the target region: 
        mean.bias[mm, i,] <- apply(bias.region, 3, mean, na.rm = TRUE) ## [n.start.months, n.regions, n.for.months]
        mean.corr[mm, i,] <- apply(corr.region, 3, mean, na.rm = TRUE)
        # mean.rpss[mm, i,] <- apply(rpss.region, 1, mean, na.rm = TRUE)
        # mean.crpss[mm, i,] <- apply(crpss.region, 1, mean, na.rm = TRUE)
        ### Replacing 1 by 3 for error: number of items to replace is not a multiple of replacement length ###
        mean.rpss[mm, i,] <- apply(rpss.region, 3, mean, na.rm = TRUE)
        mean.crpss[mm, i,] <- apply(crpss.region, 3, mean, na.rm = TRUE)
        
      }
    }
    
    ## Set values more negative of -1 to -1
    mean.crpss[which(mean.crpss < -1)] <- -1
    mean.rpss[which(mean.rpss < -1)] <- -1
    
    ## Round the scores
    mean.bias <- round(mean.bias, n.dec)
    mean.corr <- round(mean.corr, 2)
    mean.crpss <- round(mean.crpss, 2)
    mean.rpss <- round(mean.rpss, 2)
    
    ## For snow variables, remove the Tropics region
    shortertable <- FALSE
    if(var.name == "Snowfall" || var.name == "Snow density" || var.name == "Snow depth" || var.name == "Sea-ice cover"){
      mean.bias <- mean.bias[,-1,]
      mean.corr <- mean.corr[,-1,]
      mean.crpss <- mean.crpss[,-1,]
      mean.rpss <- mean.rpss[,-1,]
      region.names.figure <- region.names.figure[-1]
      n.regions <- 2
      shortertable <- TRUE
    }
    
    data <- list(mean.bias = mean.bias, mean.corr = mean.corr, mean.rpss = mean.rpss, mean.crpss= mean.crpss, 
                 region.names = region.names, region.names.figure = region.names.figure, n.regions = n.regions, 
                 n.start.months = n.start.months, n.for.months = n.for.months, n.dec = n.dec)
    
    return(data)
    
}